/**
 * @file uart.c
 * @author Bianca Forkel
 */

#include "inc/libuart.h"

int uart_filestream = -1;

int uart_setup(const char *porta) {
    strcpy(port, porta);
    uart_filestream = open(port, O_RDWR | O_NOCTTY | O_NDELAY);
    if (uart_filestream < 0) {
        fprintf(stderr, "[%s] Error opening port: %s\n", port, strerror(errno));
    }

    struct termios tty;
    memset(&tty, 0, sizeof tty);
    if (tcgetattr(uart_filestream, &tty) != 0) {
        fprintf(stderr, "[%s] Error from tcgetattr: %s\n", port, strerror(errno));
    }

    // Baud Rate
    cfsetospeed(&tty, (speed_t) B115200);
    cfsetispeed(&tty, (speed_t) B115200);

    // TTY settings: 8N1
    tty.c_cflag &= ~PARENB; //no parity
    tty.c_cflag &= ~CSTOPB; // 1 stop bit
    tty.c_cflag &= ~CSIZE;
    tty.c_cflag |= CS8; // 8 data bits

    tty.c_cflag &= ~CRTSCTS;
    tty.c_cc[VMIN] = 1;
    tty.c_cc[VTIME] = 5;
    tty.c_cflag |= CREAD | CLOCAL;

    cfmakeraw(&tty);
    tcflush(uart_filestream, TCIFLUSH);
    if (tcsetattr(uart_filestream, TCSANOW, &tty) != 0) {
        fprintf(stderr, "[%s] Error from tcsetattr: %s\n", port, strerror(errno));
    }
    return uart_filestream;
}

int uart_send_byte(char data, int printt) {
    if (uart_filestream != -1) {
        int n_written = 0;
        n_written = write(uart_filestream, &data, 1);
        if (n_written < 0) {
            if (printt)
                fprintf(stderr, "[%s] Error sending byte: %s\n", port, strerror(errno));
        } else {
            if (printt)
                printf("[%s] Sent byte %#02x\n", port, data);
            return 0;
        }
    }
    return -1;
}

int uart_send_string(char *str, int length, int printt) {
    if (uart_filestream != -1) {
        int n_written = 0;
        n_written = write(uart_filestream, str, length);
        if (n_written < 0) {
            if (printt)
                fprintf(stderr, "[%s] Error sending string: %s\n", port, strerror(errno));
        } else {
            if (printt)
                printf("[%s] Sent string %s\n", port, str);
            return 0;
        }
    }
    return -1;
}

int uart_receive(char *c, int bufLen, int printt) {
    fd_set set;
    struct timeval timeout;
    int rv;
    int n_read = -1;

    if (uart_filestream != -1) {
        FD_ZERO(&set);
        FD_SET(uart_filestream, &set);

        timeout.tv_sec = 0;
        timeout.tv_usec = 10000;

        rv = select(uart_filestream + 1, &set, NULL, NULL, &timeout);
        if (rv == -1) {
            if (printt)
                perror("select");
        } else if (rv == 0) {
            if (printt)
                perror("timeout");
        } else {
            n_read = read(uart_filestream, (void *) c, bufLen);
        }

        if (n_read < 0) {
            if (printt)
                fprintf(stderr, "[%s] Error receiving byte: %s\n", port, strerror(errno));
            n_read = 0;
        } else if (n_read == 0) {
            if (printt)
                printf("[%s] No data to receive \n", port);
        } else {
            c[n_read] = '\0';
            if (printt) {
                printf("[%s] %i Bytes received: 0x", port, n_read);
                int i;
                for (i = 0; i < n_read; i++)
                    printf("%02x", c[i]);
                printf("\n");
            }
        }
    }
    return n_read;
}

void uart_close() {
    close(uart_filestream);
}
